@@ -36,9 +36,10 @@ class BaseConnection(connection.Connection):
     DO_HANDSHAKE = True
     WARN_ABOUT_IOLOOP = False
 
-    def __init__(self, parameters=None,
-                       on_open_callback=None,
-                       stop_ioloop_on_close=True):
+    def __init__(self,
+                 parameters=None,
+                 on_open_callback=None,
+                 stop_ioloop_on_close=True):
         """Create a new instance of the Connection object.
 
         :param pika.connection.Parameters parameters: Connection parameters
@@ -58,6 +59,7 @@ class BaseConnection(connection.Connection):
         self.event_state = self.base_events
         self.socket = None
         self.write_buffer = None
+        self._remaining_attempts = 0
         super(BaseConnection, self).__init__(parameters, on_open_callback)
 
     def add_timeout(self, deadline, callback_method):
@@ -97,40 +99,47 @@ class BaseConnection(connection.Connection):
         """Connect to the RabbitMQ broker"""
         LOGGER.debug('Connecting the adapter to the remote host')
         reason = 'Unknown'
-        remaining_attempts = self.params.connection_attempts
-        while remaining_attempts:
-            remaining_attempts -= 1
-            try:
-                self._create_and_connect_to_socket()
-                return
-            except socket.timeout:
-                reason = 'timeout'
-            except socket.error, err:
-                LOGGER.error('socket error: %s', err[-1])
-                reason = err[-1]
-                self.socket.close()
+        self._remaining_attempts -= 1
+        try:
+            self._create_and_connect_to_socket()
+            return
+        except socket.timeout:
+            reason = 'timeout'
+        except socket.error, err:
+            LOGGER.error('socket error: %s', err[-1])
+            reason = err[-1]
+            self.socket.close()
 
-            if remaining_attempts:
-                LOGGER.warning('Could not connect due to "%s," retrying in %i sec',
-                               reason, self.params.retry_delay)
+        if self._remaining_attempts:
+            LOGGER.warning('Could not connect due to "%s," retrying in %i sec',
+                           reason, self.params.retry_delay)
+
+            if hasattr(self, 'ioloop') and self.ioloop:
+                LOGGER.debug('Using ioloop to wait on retry')
+                self.ioloop.add_timeout(self.params.retry_delay,
+                                        self._adapter_connect)
+            else:
                 time.sleep(self.params.retry_delay)
-
-        LOGGER.error('Could not connect: %s', reason)
-        raise exceptions.AMQPConnectionError(self.params.connection_attempts)
+                self._adapter_connect()
+        else:
+            LOGGER.error('Could not connect: %s', reason)
+            raise \
+                exceptions.AMQPConnectionError(self.params.connection_attempts)
 
     def _adapter_disconnect(self):
         """Invoked if the connection is being told to disconnect"""
         #self.socket.shutdown(socket.SHUT_RDWR)
-        self.socket.close()
+        if self.socket:
+            self.socket.close()
         self.socket = None
         self._check_state_on_disconnect()
         self._handle_ioloop_stop()
 
     def _check_state_on_disconnect(self):
-        """
-        Checks to see if we were in opening a connection with RabbitMQ when
+        """Checks to see if we were in opening a connection with RabbitMQ when
         we were disconnected and raises exceptions for the anticipated
         exception types.
+
         """
         if self.connection_state == self.CONNECTION_PROTOCOL:
             LOGGER.error('Incompatible Protocol Versions')
@@ -144,7 +153,9 @@ class BaseConnection(connection.Connection):
                          "a probable permission error when accessing a virtual "
                          "host")
             raise exceptions.ProbableAccessDeniedError
-        else:
+        elif self.is_open:
+            LOGGER.warning("Socket closed when connection was open")
+        elif not self.is_closing:
             LOGGER.warning('Unknown state on disconnect: %i',
                            self.connection_state)
 
@@ -275,6 +286,10 @@ class BaseConnection(connection.Connection):
             LOGGER.error('Received events on closed socket: %d', fd)
             return
 
+        if events & self.WRITE:
+            self._handle_write()
+            self._manage_event_state()
+
         if not write_only and (events & self.READ):
             self._handle_read()
 
@@ -282,10 +297,6 @@ class BaseConnection(connection.Connection):
             LOGGER.error('Error event %r, %r', events, error)
             self._handle_error(error)
 
-        if events & self.WRITE:
-            self._handle_write()
-            self._manage_event_state()
-
     def _handle_read(self):
         """Read from the socket and call our on_data_available with the data."""
         try:
@@ -333,6 +344,7 @@ class BaseConnection(connection.Connection):
         self.base_events = self.READ | self.ERROR
         self.event_state = self.base_events
         self.socket = None
+        self._remaining_attempts = self.params.connection_attempts
 
     def _manage_event_state(self):
         """Manage the bitmask for reading/writing/error which is used by the
