@@ -120,7 +120,7 @@ class BlockingConnection(base_connection.BaseConnection):
 
         value = {'deadline': time.time() + deadline,
                  'callback': callback_method}
-        timeout_id = hash(frozenset(value))
+        timeout_id = hash(frozenset(value.items()))
         self._timeouts[timeout_id] = value
         return timeout_id
 
